import java.awt.*;
import java.awt.event.*;

class Main extends Frame implements MouseMotionListener {
    Rectangle box = new Rectangle(0, 0, 30, 30);
    int curCursor = DEFAULT_CURSOR;
    
    Main() {
        super("MouseMotionListener Example");

        addMouseListener(new MouseEventHandler());
        addMouseMotionListener(this);
        setSize(200, 200);
        addNotify();
        box.x = getInsets().left;
        box.y = getInsets().top;
        show();
    }

    public void paint(Graphics g) {
        g.setColor(Color.red);
        g.fillRect(box.x, box.y, box.width, box.height);
    }

    public void mouseDragged(MouseEvent evt) {
        if (downPt != null) {
            Insets insets = getInsets();
            
            // Keep the box inside the frame inset area.
            box.x = Math.max(insets.left, Math.min(evt.getX()-downPt.x, 
                getSize().width-box.width-insets.right));
            box.y = Math.max(insets.top, Math.min(evt.getY()-downPt.y, 
                getSize().height-box.height-insets.bottom));
            repaint();
        }
    }

    public void mouseMoved(MouseEvent evt) {
        int oldCursor = curCursor;
        
        if (box.contains(evt.getX(), evt.getY())) {
            curCursor = Cursor.HAND_CURSOR;
        } else {
            curCursor = Cursor.DEFAULT_CURSOR;
        }
        if (oldCursor != curCursor) {
            setCursor(Cursor.getPredefinedCursor(curCursor));
        }
    }

    // Point where the mouse button was pressed relative to the box's origin.
    Point downPt;
    class MouseEventHandler extends MouseAdapter {
        public void mousePressed(MouseEvent evt) {
            if (curCursor == Cursor.HAND_CURSOR) {
                evt.translatePoint(-box.x, -box.y);
                downPt = evt.getPoint();
            }
        }
        public void mouseReleased(MouseEvent evt) {
            downPt = null;
        }
    }
    
    public static void main(String args[]) {
        new Main();
    }
}
