import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;

class Main extends Frame {
    Image image;
    Main(String filename) {
        super("setAnimate Example");
        image = new Animator().getImage(getToolkit().getImage(filename));

        // Set the initial frame size and show the image.
        setSize(300, 390);
        show();
    }
    
    public void paint(Graphics g) {
        update(g);
    }
    public void update(Graphics g) {
        g.drawImage(image, getInsets().left, getInsets().top, this);
    }

    public static void main(String[] args) {
        if (args.length == 1) {
            new Main(args[0]);
        } else {
            System.err.println("Usage: java Main <image file>");
        }
    }
}

class Animator extends Thread {
    PixelGrabber pg;
    MemoryImageSource mis;
    byte[] oldPixels;
    byte[] newPixels;

    Animator() {
    }
    Image getImage(Image image) {
        pg = new PixelGrabber(image, 0, 0, -1, -1, false);
        try {
            pg.grabPixels();
        } catch (Exception e) {
            e.printStackTrace();
        }

        // We only handle 8-bit images.
        if (!(pg.getPixels() instanceof byte[])) {
            return null;
        }

        oldPixels = (byte[])pg.getPixels();

        // Start the animation.
        start();

        newPixels = new byte[oldPixels.length];
        mis = new MemoryImageSource(pg.getWidth(), pg.getHeight(), 
            pg.getColorModel(), newPixels, 0, pg.getWidth());
        mis.setAnimated(true);
        mis.setFullBufferUpdates(true);
        return Toolkit.getDefaultToolkit().createImage(mis);
    }

    int random(int r) {
        return (int)Math.floor(Math.random()*r);
    }

    public void run() {
        int w = pg.getWidth();
        int h = pg.getHeight();
        int[] x = new int[h];
        int inc = w/20;
        boolean done = false;
        
        try {
            while (!done) {
                done = true;

                // Modify buffer.
                for (int i=0; i<h; i++) {
                    if (x[i] < w/2 && random(3) == 0) {
                        // From the left.
                        int count = Math.min(inc, w-inc);
                        System.arraycopy(oldPixels, i*w+x[i], 
                            newPixels, i*w+x[i], count);

                        // From the right.
                        count = Math.min(inc, w-inc);
                        System.arraycopy(oldPixels, (i+1)*w-x[i]-count, 
                            newPixels, (i+1)*w-x[i]-count, count);
                        x[i] += inc;
                    }
                    if (x[i] < w/2) {
                        done = false;
                    }
                }

                // Send the new pixels to the consumers.
                mis.newPixels(0, 0, pg.getWidth(), pg.getHeight());
                Thread.sleep(33);
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
}
