import java.awt.*;
import java.awt.image.*;
import java.awt.event.*;

class Main extends Frame implements ActionListener {
    Main(String filename) {
        super("Component.contains()");

        // Create image button and listen for events.
        ImageComponent c = new ImageComponent(filename);
        c.addActionListener(this);

        // Layout and show components.
        add(c, BorderLayout.CENTER);
        pack();
        show();
    }

    public void actionPerformed(ActionEvent evt) {
        System.out.println(evt);
    }

    public static void main(String[] args) {
        if (args.length != 1) {
            System.err.println("Usage: java Main <image-file>");
            System.exit(1);
        }
        new Main(args[0]);
    }
}

class ImageComponent extends Component {
    Dimension imageDim;
    Image image;
    int transparentPixel;
    byte[] pixels;
 
    ImageComponent(String filename) {
        // Retrieve the image from a file.
        image = getToolkit().getImage(filename);

        // Use -1 for the width and height to grab everything.
        PixelGrabber pg = new PixelGrabber(image, 0, 0, -1, -1, false);

        // Get the pixels.
        pg.startGrabbing(); 
        try {
            pg.grabPixels();
        } catch (Exception e) {
            e.printStackTrace();
        }
        imageDim = new Dimension(pg.getWidth(), pg.getHeight());

        // If the color map is an index color map, get the 
        // pixels and the transparent pixel.
        if (pg.getColorModel() instanceof IndexColorModel) {
            pixels = (byte[])pg.getPixels();
            transparentPixel = 
                ((IndexColorModel)pg.getColorModel()).getTransparentPixel();
        }

        // Enable mouse events.
        addMouseListener(new MouseEventHandler());

        // Set the default cursor to be a hand cursor.
        setCursor(Cursor.getPredefinedCursor(Cursor.HAND_CURSOR));
    }
        
    public boolean contains(int x, int y) {
        // Return true on if the cursor is over non-transparent pixels.
        if (x >= 0 && x < imageDim.width
              && y >= 0 && y < imageDim.height) {
            return pixels == null ||
                pixels[x+y*imageDim.width] != transparentPixel;
        }
        return false;
    }

    public Dimension getPreferredSize() {
        return imageDim;
    }

    public void paint(Graphics g) {
        g.setColor(Color.pink);
        g.fillRect(0, 0, getSize().width, getSize().height);
        g.drawImage(image, 0, 0, this);
    }

    class MouseEventHandler extends MouseAdapter {
        public void mousePressed(MouseEvent evt) {
            if (actionListener != null) {
                actionListener.actionPerformed(    
                    new ActionEvent(ImageComponent.this,
                        ActionEvent.ACTION_PERFORMED, null));
            }
        }
    }

    // Action listener support methods.
    transient ActionListener actionListener;
    public synchronized void addActionListener(ActionListener l) {
        actionListener = AWTEventMulticaster.add(actionListener, l);
    }

    public synchronized void removeActionListener(ActionListener l) {
        actionListener = AWTEventMulticaster.remove(actionListener, l);
    }
}
