import java.awt.*;
import java.awt.event.*;

class Main extends Frame implements ActionListener {
    Main() {
        super("printAll Example");

        // Create the paint and print buttons and listen for events.
        Button b = new Button("Paint Preview");
        b.addActionListener(this);
        add(b);
        b = new Button("Print Preview");
        b.addActionListener(this);
        add(b);

        // Now create and add a bunch of miscellaneous components 
        // in no particular order.
        List l = new List();
        l.addItem("item 0");
        l.addItem("item 1");
        add(l);
        Choice ch = new Choice();
        ch.addItem("item 0");
        ch.addItem("item 1");
        add(ch);
        add(new TextField("TextField"));
        Panel p = new Panel(new FlowLayout());
        p.add(new Checkbox("Checkbox"));
        // Add a lightweight component.
        p.add(new ImageComponent("duke.gif"));
        add(p);

        // Layout and show.
        setLayout(new FlowLayout());
        pack();
        show();
    }

    // Color just the inset area.
    public void paint(Graphics g) {
        Insets insets = getInsets();

        g.setColor(Color.lightGray);
        g.fillRect(insets.left, insets.top, 
            getSize().width-insets.left-insets.right,
            getSize().height-insets.top-insets.bottom);

        // Don't forget to paint the lightweights.
        super.paint(g);
    }

    public void actionPerformed(ActionEvent evt) {
        // Create an off-screen image and a graphics context on it.
        Image image = createImage(getSize().width, getSize().height);
        Graphics g = image.getGraphics();

        // A workaround for a clipping bug in 
        // Container.paintComponents() and Container.printComponents().
        g.setClip(0, 0, Short.MAX_VALUE, Short.MAX_VALUE);

        // Paint or print the components in the graphics context.
        if (evt.getActionCommand().equals("Paint Preview")) {
            paintAll(g);

            // The following is almost the same except that the frame's
            // paint() method is not be called.
            //paintComponents(g);

            // The following only invokes the paint() method of the frame
            // and any lightweight children.
            //paint(g);

            // Create the viewer to see the results.
            new Previewer(image, "Paint");
        } else if (evt.getActionCommand().equals("Print Preview")) {
            printAll(g);

            // The following is almost the same except that the frame's
            // print() method is not be called.
            //printComponents(g);

            // The following only invokes the print() method of the frame
            // and any lightweight children.
            //print(g);

            // Create the viewer to see the results.
            new Previewer(image, "Print");
        }
        g.dispose();

    }

    public static void main(String[] args) {
        new Main();
    }
}

class ImageComponent extends Component {
    Image image;
    ImageComponent(String filename) {
        image = getToolkit().getImage(filename);

        // Load the image with a media tracker because
        // we need the dimensions immediately.
        MediaTracker tracker = new MediaTracker(this);
        try {
            tracker.addImage(image, 0);
            tracker.waitForAll(0);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
        
    public Dimension getPreferredSize() {
        return new Dimension(image.getWidth(null), image.getHeight(null));
    }

    public void paint(Graphics g) {
        g.drawImage(image, 0, 0, this);
    }
}

class Previewer extends Frame {
    Image image;

    Previewer(Image image, String title) {
        super(title + " Viewer");
        int w = image.getWidth(null);
        int h = image.getHeight(null);

        this.image = image;

        // This call creates the frame's peers.  
        // Without this call, getInsets() will return invalid values.
        addNotify();

        // When the frame's insets.
        Insets insets = getInsets();

        // Adjust the dimensions of the frame to account for the insets.
        w += insets.left + insets.right;
        h += insets.top + insets.bottom;

        // Size and show the viewer.
        setSize(w, h);
        show();
    }

    // Shows the image.
    public void paint(Graphics g) {
        Insets insets = getInsets();

        g.drawImage(image, insets.left, insets.top, this);
    }
}
