import java.awt.*;
import java.awt.event.*;

class Main extends Frame {
    CheckboxMenuItem testCB = new CheckboxMenuItem("Test");

    Main() {
        super("Container Example");
        MenuBar mb = new MenuBar();
        ActionListener actionListener = new ActionEventHandler();
        MouseEventHandler mouseListener = new MouseEventHandler();

        // Initialize Edit Menu 
        Menu m = new Menu("Edit");
        m.add("Remove All");
        testCB.addItemListener(new ItemEventHandler());
        m.add(testCB);
        mb.add(m);
        m.addActionListener(actionListener);

        // Initialize Add Menu 
        m = new Menu("Add");
        m.add("Button");
        m.add("Checkbox");
        m.add("Choice");
        m.add("List");
        m.add("Scrollbar");
        m.add("TextArea");
        m.add("TextField");
        m.add("Lightweight");
        mb.add(m);

        // Set the menubar.
        setMenuBar(mb);

        // Listen for events.
        m.addActionListener(actionListener);
        addMouseListener(mouseListener);
        addMouseMotionListener(mouseListener);


        // Remove the default layout manager.
        setLayout(null);

        setSize(400, 200);
        show();
    }

    Component dragging;
    Component stretching;
    Point offset;

    class MouseEventHandler extends MouseAdapter 
        implements MouseMotionListener {
        public void mousePressed(MouseEvent evt) {
            int x = evt.getX();
            int y = evt.getY();
            Component c = getComponentAt(x, y);

            if (c != null && c != Main.this) {
                if (evt.isShiftDown() && evt.isControlDown()) {
                    remove(c);
                } else if (evt.isAltDown()) {
                    stretching = c;

                    // Bring to front.
                    remove(c);
                    add(c, 0);
                } else {
                    offset = new Point(x-c.getLocation().x, 
                                       y-c.getLocation().y);
                    dragging = c;

                    // Bring to front.
                    remove(c);
                    add(c, 0);
                }
            }
        }
        public void mouseReleased(MouseEvent evt) {
            stretching = dragging = null;
        }
        public void mouseDragged(MouseEvent evt) {
            int x = evt.getX();
            int y = evt.getY();
            if (dragging != null) {
                dragging.setLocation(x-offset.x, y-offset.y);
            } else if (stretching != null) {
                stretching.setSize(x-stretching.getBounds().x, 
                                   y-stretching.getBounds().y);
            }
        }
        public void mouseMoved(MouseEvent evt) {
        }
    }

    int newX, newY;
    class ActionEventHandler implements ActionListener {
        public void actionPerformed(ActionEvent evt) {
            newX = Math.max(getInsets().left, newX);
            newY = Math.max(getInsets().top, newY);
            Component c = null;
            String arg = evt.getActionCommand();

            if ("Button".equals(arg)) {
                add(c = new Button("Button"));
            } else if ("Checkbox".equals(arg)) {
                add(c = new Checkbox("Checkbox"));
            } else if ("Choice".equals(arg)) {
                add(c = new Choice());
            } else if ("List".equals(arg)) {
                add(c = new List());
            } else if ("Scrollbar".equals(arg)) {
                add(c = new Scrollbar());
            } else if ("TextArea".equals(arg)) {
                add(c = new TextArea("TextArea", 3, 20));
            } else if ("TextField".equals(arg)) {
                add(c = new TextField("TextField"));
            } else if ("Lightweight".equals(arg)) {
                add(c = new Lightweight());
            } else if ("Remove All".equals(arg)) {
                removeAll();
                // The following is needed to clear the lightweights.
                repaint();
            } 
            if (c != null) {
                Dimension d = c.getPreferredSize();

                c.setBounds(newX, newY, d.width, d.height);
                c.setEnabled(testCB.getState());
                newX += 20;
                newY += 20;
                if (newX > getSize().width*3/4) {
                    newX = 0;
                }
                if (newY > getSize().height*3/4) {
                    newY = 0;
                }
            }
        }
    }

    class ItemEventHandler implements ItemListener {
        public void itemStateChanged(ItemEvent evt) {
            Component[] all = getComponents();
            boolean newState = (evt.getStateChange() == ItemEvent.SELECTED);

            // Enable all the components.
            for (int i=0; i<all.length; i++) {
                all[i].setEnabled(newState);
            }
        }
    }

    static public void main(String[] args) {
        new Main();
    }
}

class Lightweight extends Component implements FocusListener {
    boolean hasFocus;
    Lightweight() {
        // Listen for focus events.
        addFocusListener(this);
        setSize(80, 80);
    }
    public void paint(Graphics g) {
        if (hasFocus) {
            g.setColor(Color.green);
        } else {
            g.setColor(Color.red);
        }
        g.fillOval(0, 0, getSize().width, getSize().height);
        g.setColor(Color.black);
        g.drawOval(0, 0, getSize().width-1, getSize().height-1);
    }

    public Dimension getPreferredSize() {
        return getSize();
    }

    public boolean isFocusTraversable() {
	return true;
    }

    public boolean contains(int x, int y) {
        int midX = getSize().width/2;
        int midY = getSize().height/2;
 
        // This formula doesn't really work for ovals.
        return (midX-x)*(midX-x) + (midY-y)*(midY-y) <= midX*midX;
    }

    public void focusGained(FocusEvent evt) {
	hasFocus = true;
        repaint();
    }
    public void focusLost(FocusEvent evt) {
	hasFocus = false;
        repaint();
    }
}
