import java.awt.*;
import java.awt.event.*;
import java.io.*;

class Main extends Frame implements FilenameFilter, ActionListener {
    FileDialog fd;
    TextField tfDirectory = new TextField();
    TextField tfFile = new TextField();
    TextField tfFilter = new TextField();

    Main() {
        super("FileDialog Example");
        Button lb = new Button("Load");
        lb.addActionListener(this);
        Button sb = new Button("Save");
        sb.addActionListener(this);
        add(lb, BorderLayout.WEST);
        add(sb, BorderLayout.EAST);

        Panel p = new Panel(new GridBagLayout());
        addRow(p, new Label("directory:", Label.RIGHT), tfDirectory);
        addRow(p, new Label("file:", Label.RIGHT), tfFile);
        addRow(p, new Label("filter:", Label.RIGHT), tfFilter);

        add(p, BorderLayout.SOUTH);
        pack();
        show();
    }

    // Adds a row in a gridbag layout where the c2 is stretchy
    // and c1 is not.
    void addRow(Container cont, Component c1, Component c2) {
        GridBagLayout gbl = (GridBagLayout)cont.getLayout();
        GridBagConstraints c = new GridBagConstraints();
        Component comp;

        c.fill = GridBagConstraints.BOTH;
        cont.add(c1);
        gbl.setConstraints(c1, c);

        c.gridwidth = GridBagConstraints.REMAINDER;
        c.weightx = 1.0;
        cont.add(c2);
        gbl.setConstraints(c2, c);
    }

    public boolean accept(File dir, String name) {
        if (fd.getMode() == FileDialog.LOAD) {
            return name.lastIndexOf(tfFilter.getText()) > 0;
        }
        return true;
    }

    public void actionPerformed(ActionEvent evt) {
        String what = evt.getActionCommand();
        boolean load = "Load".equals(what);

        if ("Load".equals(what)) {
            fd = new FileDialog(this, null, FileDialog.LOAD);
        } else if ("Save".equals(what)) {
            fd = new FileDialog(this, null, FileDialog.SAVE);
        } else {
            return;
        }
        
        fd.setDirectory(tfDirectory.getText());
        fd.setFile(tfFile.getText());
        fd.setFilenameFilter(Main.this);

        fd.show();

        // Update text fields with input from dialogs
        tfDirectory.setText(fd.getDirectory());
        tfFile.setText(fd.getFile());
    }

    static public void main(String[] args) {
        new Main();
    }

}
