import java.awt.*;
import java.awt.event.*;
import java.util.Vector;

class Main extends Frame implements ItemListener {
    Main() {
        super("ItemSelectable Example");
        ButtonPanel bp = new ButtonPanel();

        bp.add(new Button("first"));
        bp.add(new Button("second"));
        bp.add(new Button("third"));
        bp.add(new Button("fourth"));
        bp.add(new Button("fifth"));

        add(bp, BorderLayout.SOUTH);
        bp.addItemListener(this);

        pack();
        show();
    }

    public void itemStateChanged(ItemEvent evt) {
        ItemSelectable src = (ItemSelectable)evt.getSource();
        Object[] selected = src.getSelectedObjects();

        System.out.println("item event: " + evt);
        for (int i = 0; i <selected.length; i++) {
            System.out.println("selected: " + selected[i]);
        }
    }

    public static void main(String[] args) {
        new Main();
    }
}

class ButtonPanel extends Container implements ItemSelectable, ActionListener {
    transient ItemListener itemListener;
    int selected = -1;
    Vector vec = new Vector();

    ButtonPanel () {
        super();

        setLayout(new GridLayout(1, -1));
        enableEvents(AWTEvent.ITEM_EVENT_MASK);
    }

    void add(Button b) {
        super.add(b);

        b.addActionListener(this);
        vec.addElement(b);
        if (selected == -1) {
            selected = 0;
            b.setEnabled(false);
        } else {
            b.setEnabled(true);
        }
    }

    public void actionPerformed(ActionEvent evt) {
        Object src = evt.getSource();
        int where = vec.indexOf(src);

        if (where >= 0) {
            Button oldb = (Button)vec.elementAt(selected);
            oldb.setEnabled(true);
            Button newb = (Button)vec.elementAt(where);
            newb.setEnabled(false);
            selected = where;

            ItemEvent oldEvt = 
                new ItemEvent(this, ItemEvent.ITEM_STATE_CHANGED, 
                              oldb, ItemEvent.DESELECTED);
            // dispatch item event for old button
            processEvent(oldEvt);

            ItemEvent newEvt = 
                new ItemEvent(this, ItemEvent.ITEM_STATE_CHANGED, 
                              newb, ItemEvent.SELECTED);
            // dispatch item event for new button
            processEvent(newEvt);
        }
    }

    public Object[] getSelectedObjects() {
        if (selected >= 0) {
            Object[]  answer = new Object[1];
            answer[0] = vec.elementAt(selected);
            return answer;
        } else {
            return new Object[0];
        }
    }
    public synchronized void addItemListener(ItemListener l) {
        itemListener = AWTEventMulticaster.add(itemListener, l);
    }
    public void removeItemListener(ItemListener l) {
        itemListener = AWTEventMulticaster.remove(itemListener, l);
    }

    protected void processEvent(AWTEvent e) {
        if (e instanceof ItemEvent) {
            processItemEvent((ItemEvent)e);
            return;
        }
        super.processEvent(e);
    }

    protected void processItemEvent(ItemEvent e) {
        if (itemListener != null) {
            itemListener.itemStateChanged(e);
        }
    }
}
