import java.awt.*;
import java.awt.event.*;

class Main extends Frame implements ActionListener {
    Menu menu = new Menu("Font Size");

    Main() {
        super("processActionEvent Example");

        setLayout(new FlowLayout());
        MenuItem b;
        menu.add(b = new MainMenuItem("SmallFont"));
        b.addActionListener(this);
        menu.add(b = new MainMenuItem("LargeFont"));
        b.addActionListener(this);

        MenuBar mb = new MenuBar();
        mb.add(menu);
        setMenuBar(mb);

        setSize(200, 200);
        show();
    }

    public void actionPerformed(ActionEvent evt) {
        String arg = evt.getActionCommand();
        Font f = null;

        if ("SmallFont".equals(arg)) {
            f = new Font("Serif", Font.PLAIN, 12);
        } else if ("LargeFont".equals(arg)) {
            f = new Font("Serif", Font.PLAIN, 30);
        }
        if (f != null) {
            SetFontEvent fevt = new SetFontEvent(this, f);
            for (int i=0; i<menu.getItemCount(); i++) {
                MainMenuItem c = (MainMenuItem)menu.getItem(i);
                c.dispatchEvent(fevt);
            }
        }
    }

    public static void main(String[] args) {
        new Main();
    }
}

// Define new event for setting font of menu item
// Must be subclass of ActionEvent to get past dispatchEvent's filter

class SetFontEvent extends ActionEvent {
    Font font;
    public SetFontEvent(Object source, Font f) {
        // Must use ACTION_PERFORMED to get event pass 
        // dispatchEvent's filter
        super(source, ActionEvent.ACTION_PERFORMED, null);
        font = f;
    }
    public Font getFont() {
        return font;
    }
}

class MainMenuItem extends MenuItem {
    MainMenuItem(String label) {
        super(label);
    }

    // Override to handle special new event (SetFontEvent)
    protected void processActionEvent(ActionEvent evt) {
        if (evt instanceof SetFontEvent) {
            // Change font as requested
            processSetFontEvent((SetFontEvent) evt);
        } else {
            super.processActionEvent(evt);
        }
    }

    protected void processSetFontEvent(SetFontEvent evt) {
        // perform font change on this menu item
        setFont(evt.getFont());
    }
}
