import java.awt.*;
import java.awt.event.*;

class Main extends Frame implements Runnable {
    FakeDesktop desktop = new FakeDesktop();

    Main() {
        super("SystemColor Example");

        // Set up first window.
        FakeWindow w = new FakeWindow(
            SystemColor.WINDOW, SystemColor.WINDOW_TEXT, true);
        w.setBounds(40, 40, 200, 100);
        desktop.add(w);

        // Set up second window.
        w = new FakeWindow(
            SystemColor.WINDOW, SystemColor.WINDOW_TEXT, false);
        w.setBounds(20, 10, 200, 100);
        desktop.add(w);

        // Set up first button.
        FakeButton b = new FakeButton(true);
        b.setBounds(20, 150, 80, 25);
        desktop.add(b);

        // Set up second button.
        b = new FakeButton(false);
        b.setBounds(20, 185, 80, 25);
        desktop.add(b);

        // Set up list.
        FakeList l = new FakeList();
        l.setBounds(140, 150, 100, 70);
        desktop.add(l);

        // Set up the desktop.
        add(desktop, BorderLayout.CENTER);

        // Set up control panel.
        add(new ControlPanel(this), BorderLayout.EAST);
        setSize(590, 260);

        // Show the whole thing.
        show();

        // Start the blinking thread.
        (new Thread(this)).start();
    }

    static Color[] colors = {
        SystemColor.desktop, SystemColor.activeCaption,
        SystemColor.activeCaptionText, SystemColor.activeCaptionBorder,
        SystemColor.inactiveCaption, SystemColor.inactiveCaptionText,
        SystemColor.inactiveCaptionBorder, SystemColor.window,
        SystemColor.windowBorder, SystemColor.windowText,
        SystemColor.menu, SystemColor.menuText,
        SystemColor.text, SystemColor.textText,
        SystemColor.textHighlight, SystemColor.textHighlightText,
        SystemColor.textInactiveText, SystemColor.control,
        SystemColor.controlText, SystemColor.controlHighlight,
        SystemColor.controlLtHighlight, SystemColor.controlShadow,
        SystemColor.controlDkShadow, SystemColor.scrollbar,
        SystemColor.info, SystemColor.infoText,
    };

    int curColor;
    void setCurColor(int c) {
        curColor = c;
    }

    public void run() {
        Color[] savedColors = (Color[])colors.clone();
        boolean on = false;
        int c = curColor;

        while (true) {
            // curColor changed so restore it before
            // blinking new color.
            if (c != curColor) {
                colors[c] = savedColors[c];
                c = curColor;
            }

            try {
                if (on) {
                    colors[c] = savedColors[c];
                } else {
                    colors[c] = Color.red;
                }
                on = !on;
                repaintAll(desktop);
                Thread.sleep(500);
            } catch (Exception e) {
            }
        }
    }

    public void update(Graphics g) {
        super.paint(g);
    }

    public void repaintAll(Component c) {
        if (c instanceof Container) {
            Container cont = (Container)c;
            Component[] children = cont.getComponents();

            for (int i=0; i<children.length; i++) {
                repaintAll(children[i]);
            }
        }
        c.repaint();
    }
        
    public static void main(String[] args) {
        new Main();
    }
}

class FakeDesktop extends Container {
    FakeDesktop() {
        setLayout(null);

        // Listen for events.
        addMouseListener(new MouseEventHandler());
        addMouseMotionListener(new MouseMotionEventHandler());
    }

    public void paint(Graphics g) {
        update(g);
    }

    Image bbuf;
    Graphics bbufG;
    public void update(Graphics g) {
        FontMetrics fm = g.getFontMetrics();
        int w = getSize().width;
        int h = getSize().height;

        if (bbuf == null
                || bbuf.getWidth(null) < w
                || bbuf.getHeight(null) < h) {
            bbuf = createImage(w, h);
            bbufG = bbuf.getGraphics();
        }
        bbufG.setColor(Main.colors[SystemColor.DESKTOP]);
        bbufG.fillRect(0, 0, w, h);

        bbufG.clipRect(0, 0, w, h);
        // Paint all of the lightweight's children.
        super.paint(bbufG);

        // Paint Info-tag.
        if (mousePt != null) {
            w = fm.stringWidth("Info-tag")+4;
            h = fm.getHeight()+4;

            // Paint Info-tag background color.
            bbufG.setColor(Main.colors[SystemColor.INFO]);
            bbufG.fillRect(mousePt.x, mousePt.y, w, h);

            // Paint Info-tag text.
            bbufG.setColor(Main.colors[SystemColor.INFO_TEXT]);
            bbufG.drawString("Info-tag", 
                mousePt.x+2, mousePt.y+2+fm.getAscent());
            bbufG.drawRect(mousePt.x, mousePt.y, w-1, h-1);
        }

        g.drawImage(bbuf, 0, 0, this);
    }

    // If non-null, holds current location of mouse
    Point mousePt;

    class MouseEventHandler extends MouseAdapter {
        public void mouseExited(MouseEvent evt) {
            mousePt = null;
            repaint();
        }
    }
    class MouseMotionEventHandler extends MouseMotionAdapter {
        public void mouseMoved(MouseEvent evt) {
            mousePt = evt.getPoint();
            repaint();
        }
    }
}

class ControlPanel extends Panel implements ItemListener {
    CheckboxGroup cbg = new CheckboxGroup();
    Main frame;

    ControlPanel(Main f) {
        frame = f;
        setLayout(new GridLayout(0, 2));

        for (int i=0; i<SystemColor.NUM_COLORS; i++) {
            Checkbox cb = new Checkbox(colorNames[i], i==0, cbg);

            cb.setName(""+i);
            cb.addItemListener(this);
            add(cb);
        }

    }

    public void itemStateChanged(ItemEvent evt) {
        frame.setCurColor(
            Integer.parseInt(cbg.getSelectedCheckbox().getName()));
    }

    static String[] colorNames = {
        "desktop", "activeCaption",
        "activeCaptionText", "activeCaptionBorder",
        "inactiveCaption", "inactiveCaptionText",
        "inactiveCaptionBorder", "window",
        "windowBorder", "windowText",
        "menu", "menuText",
        "text", "textText",
        "textHighlight", "textHighlightText",
        "textInactiveText", "control",
        "controlText", "controlHighlight",
        "controlLtHighlight", "controlShadow",
        "controlDkShadow", "scrollbar",
        "info", "infoText",
    };
}
