package classexample;

import java.beans.*;
import java.lang.reflect.*;

public class BeanBeanInfo extends SimpleBeanInfo {
    public EventSetDescriptor[] getEventSetDescriptors() {
        try {
            EventSetDescriptor[] esd = new EventSetDescriptor[4];

            // First form of the constructor.
            esd[0] = new EventSetDescriptor(Bean.class,
                    "jump",
                    JumpListener.class,
                    "beanJumped");
            esd[0].setUnicast(true);
            esd[0].setInDefaultEventSet(true);

            // Second form of the constructor.
            String[] listenerNames1 = {"focusGained", "focusLost"};
            esd[1] = new EventSetDescriptor(Bean.class,
                    "focus",
                    java.awt.event.FocusListener.class,
                    listenerNames1, 
                    "addFocusListener", "removeFocusListener");

            // Third form of the constructor.
            Method[] listeners2 = {
                getMethod(java.awt.event.MouseMotionListener.class, 
                    "mouseMoved", 1), 
                getMethod(java.awt.event.MouseMotionListener.class, 
                    "mouseDragged", 1),   };
            esd[2] = new EventSetDescriptor(
                    "mousemotion",
                    java.awt.event.MouseMotionListener.class,
                    listeners2, 
                    getMethod(Bean.class, "addMouseMotionListener", 1), 
                    getMethod(Bean.class, "removeMouseMotionListener", 1));

            // Fourth form of the constructor.
            MethodDescriptor[] listeners3 = {
                getMethodDesc(java.awt.event.MouseListener.class, 
                    "mouseClicked", 1), 
                getMethodDesc(java.awt.event.MouseListener.class, 
                    "mousePressed", 1),
                getMethodDesc(java.awt.event.MouseListener.class, 
                    "mouseReleased", 1),
                getMethodDesc(java.awt.event.MouseListener.class, 
                    "mouseEntered", 1),
                getMethodDesc(java.awt.event.MouseListener.class, 
                    "mouseExited", 1),   };
            esd[3] = new EventSetDescriptor(
                    "mouse",
                    java.awt.event.MouseListener.class,
                    listeners3,
                    getMethod(Bean.class, "addMouseListener", 1),
                    getMethod(Bean.class, "removeMouseListener", 1));

            return esd;
        } catch (IntrospectionException e) {
            e.printStackTrace();
        }
        return null;
    }

    Method getMethod(Class cls, String methodName, int argCount) 
            throws IntrospectionException {
        Method methods[] = cls.getMethods();

        for (int i = 0; i < methods.length; i++) {
            Method method = methods[i];

            if (method.getName().equals(methodName) 
                    && method.getParameterTypes().length == argCount) {
                return method;
            }
        }
        throw new IntrospectionException("No method \"" + methodName 
            + "\" with " + argCount + " argument");
    }

    MethodDescriptor getMethodDesc(Class cls, String methodName, int argCount) 
            throws IntrospectionException {
        return new MethodDescriptor(getMethod(cls, methodName, argCount));
    }
}
