import java.awt.*;
import java.awt.image.*;
import java.awt.event.*;
import java.util.TooManyListenersException;

class Main extends Frame implements ButtonClickListener {
    Main(String upFile, String downFile) {
        super("AWTEvent Example");
        ImageButton b = new ImageButton(getToolkit().getImage(upFile), 
                                        getToolkit().getImage(downFile), 
                                       "Hello");
        add(b, BorderLayout.CENTER); 

        try {
            b.addButtonClickListener(this);
        } catch (TooManyListenersException e) {
            e.printStackTrace();
            System.exit(-1);
        }

        setBackground(Color.lightGray);
        pack();
        show();
    }

    public void buttonClicked(ButtonClickEvent evt) {
        System.out.println(evt.toString());
        System.out.println(evt.getButtonRequest());
    }

    static public void main(String[] args) {
        if (args.length == 2) {
            new Main(args[0], args[1]);
        } else {
            System.err.println("usage: java Main <image1> <image2>");
        }
    }
}

// New subclass of AWTEvent
class ButtonClickEvent extends AWTEvent {
    String request;
    // Event id must be greater than RESERVED_ID_MAX
    static final int BUTTON_CLICK_EVENT = AWTEvent.RESERVED_ID_MAX+100;

    ButtonClickEvent(Object source, String request) {
        super(source, BUTTON_CLICK_EVENT);
        this.request = request;
        consumed = true;  // semantic event need not go to peer
    }

    String getButtonRequest() {
        return request;
    }

    public String paramString() {
        return getButtonRequest() + super.paramString();
    }
}

// Interface of listener
interface ButtonClickListener {
    public void buttonClicked(ButtonClickEvent evt);
}

// Component that fires ButtonClickEvent
class ImageButton extends Component {
    Image upImage, downImage;
    transient ButtonClickListener listener;
    String cmd;
    boolean raised = true;
    int borderWidth = 5;

    ImageButton(Image upImage, Image downImage, String cmd) {
        this.upImage = upImage;
        this.downImage = downImage;
        this.cmd = cmd;

        // Load first image to get size of button.
        MediaTracker tracker = new MediaTracker(this);
        try {
            tracker.addImage(upImage, 0);
            tracker.waitForAll(0);
        } catch (InterruptedException e) {
            e.printStackTrace();
        }
        addMouseListener(new MouseEventHandler());
    }

    public Dimension getPreferredSize() {
        return new Dimension(upImage.getWidth(null)+2*borderWidth, 
                             upImage.getHeight(null)+2*borderWidth);
    }

    public boolean contains(int x, int y) {
        return (x < upImage.getWidth(null)+2*borderWidth &&
                y < upImage.getHeight(null)+2*borderWidth);
    }

    public void paint(Graphics g) {
        Image image = (raised ? upImage : downImage); 
        g.drawImage(image, borderWidth, borderWidth, this);

        // Draw border around image
        int x = 0, y = 0;
        int w = image.getWidth(this)+2*borderWidth;
        int h = image.getHeight(this)+ 2*borderWidth;
        w -= 1;       // draw inside the rectangle.
        h -= 1;       // draw inside the rectangle.
        g.setColor(getBackground());
        for (int i=0; i<borderWidth; i++) {
            g.draw3DRect(x++, y++, w, h, raised);
            w -= 2;
            h -= 2;
        }
    }

    // Method for registering ButtonClickEvent listener
    public synchronized void addButtonClickListener(ButtonClickListener l)
        throws TooManyListenersException {
            if (listener != null) {
                throw new TooManyListenersException();
            }
            listener = l;
        }

    // Override processEvent() to handle new event type
    protected void processEvent(AWTEvent evt) {
        if (evt instanceof ButtonClickEvent) {
            processButtonClickEvent((ButtonClickEvent)evt);
            return;
        }
        super.processEvent(evt);
    }

    // Method for processing new event type
    protected void processButtonClickEvent(ButtonClickEvent evt) {
        if (listener != null) {
            listener.buttonClicked(evt);
        }
    }

    // Mouse handler for presses and releases inside this button
    // When mouse is released, fire ButtonClickEvent
    class MouseEventHandler extends MouseAdapter {
        public void mousePressed(MouseEvent evt) {
            raised = false;
            repaint();
        }
        public void mouseReleased(MouseEvent evt) {
            raised = true;
            repaint();
            ButtonClickEvent bd = new ButtonClickEvent(ImageButton.this, cmd);
            Toolkit.getDefaultToolkit().getSystemEventQueue().postEvent(bd);
        }
    }
}

